<?php

namespace app\api\controller;

use app\api\controller\base\Base;
use app\common\model\Address;
use app\common\model\Appointment as appointmentModel;
use app\common\model\AppointmentApplication;
use app\common\model\AppointmentTicket;
use time\DateHelper;

class Appointment extends Base
{
    //获取地址列表
    public function addressList()
    {
        $post = $this->request->param();
        $validate = new \think\Validate([
            ['page', 'number'],
            ['pagenum', 'number|<=:1000']
        ]);
        if (!$validate->check($post)) {
            $this->json_error('提交失败：' . $validate->getError());
        }

        $pagenum = $this->request->param('pagenum', 20, 'intval');
        $datalist = (new Address())->paginate($pagenum, true);
        if (empty($datalist)) {
            $this->json_error("没有数据");
        }
        foreach ($datalist as $key => $item) {
            $item['thumb_url'] = geturl($item->thumb, '', true);
            $datalist[$key] = $item;
        }
        $this->json_success("查询成功", $datalist);
    }


    //可预约的地址
    public function availableAddressList()
    {
        $post = $this->request->param();
        $validate = new \think\Validate([
            ['page', 'number'],
            ['pagenum', 'number|<=:1000']
        ]);
        if (!$validate->check($post)) {
            $this->json_error('提交失败：' . $validate->getError());
        }

        $where = [];
        $addressIds = (new AppointmentTicket())->where('appointment_daytime', '>=', strtotime(date('Y-m-d')))->distinct(true)->column('address_id');
        $where['id'] = ['in', $addressIds];
        $pagenum = $this->request->param('pagenum', 20, 'intval');
        $datalist = (new Address())->where($where)->paginate($pagenum, true);
        if (empty($datalist)) {
            $this->json_error("没有数据");
        }
        foreach ($datalist as $key => $item) {
            $item['thumb_url'] = geturl($item->thumb, '', true);
            $datalist[$key] = $item;
        }
        $this->json_success("查询成功", $datalist);
    }


    //按地址id获取号源信息
    public function getTicketByAddressId()
    {
        $post = $this->request->param();
        $validate = new \think\Validate([
            ['address_id', 'require|number'],
            ['daytime', 'dateFormat:Y-m-d'],
        ]);
        if (!$validate->check($post)) {
            $this->json_error('提交失败：' . $validate->getError());
        }

        $address_id = $this->request->param('address_id');
        $daytime = $this->request->param('daytime', date('Y-m-d'));

        $ticketModel = new AppointmentTicket();
        $tickets = $ticketModel->where('address_id', $address_id)->where('appointment_daytime', strtotime($daytime))->select();

        foreach ($tickets as $k => $item) {
            $appoint = $item->appointment;
            //专家信息
            $item->specialist;
            $item->specialist->head_pic = geturl($item->specialist->head_pic, '', true);
            //拆分预约时段
            $ticketPeriods = (new AppointmentApplication())->where('appointment_ticket_id', $item->id)->column('appointment_time');
            $item['morning_time_period'] = $this->setPeriodStatus(DateHelper::splitTimePeriod($appoint->morning_start_time, $appoint->morning_end_time, $appoint->morning_num), $ticketPeriods);
            $item['afternoon_time_period'] = $this->setPeriodStatus(DateHelper::splitTimePeriod($appoint->afternoon_start_time, $appoint->afternoon_end_time, $appoint->afternoon_num), $ticketPeriods);
            $item['night_time_period'] = $this->setPeriodStatus(DateHelper::splitTimePeriod($appoint->night_start_time, $appoint->night_end_time, $appoint->night_num), $ticketPeriods);
            //剩余预约号码
            $item['morning_remaining'] = count($item['morning_time_period']);
            $item['afternoon_remaining'] = count($item['afternoon_time_period']);
            $item['night_remaining'] = count($item['night_time_period']);
            $tickets[$k] = $item;
        }

        $this->json_success("查询成功", $tickets);
    }


    /**
     * @param $timePeriods array 拆分的时间段
     * @param $ticketPeriods array 已经预约的时间段
     * @return array
     */
    private function setPeriodStatus($timePeriods, $ticketPeriods)
    {
        $daytime = $this->request->param('daytime', date('Y-m-d'));
        $arr = [];
        $now = time();
        foreach ($timePeriods as $ticket => $period) {
            //排除过期时段
            $timerang = explode(' - ', $period);
            $period_end = $timerang[1];
            if ($now > strtotime($daytime . ' ' . $period_end)) {
                continue;
            }
            //标记预约状态
            if (in_array($period, $ticketPeriods)) {
//                $arr[] = ['period' => $period, 'status' => 1];
                continue;//排除
            } else {
                $arr[] = ['period' => $period, 'status' => 0];
            }
        }
        return $arr;
    }

    //按时间获取地址列表
    public function addressListByDayTime()
    {
        $ticketModel = new AppointmentTicket();
        $post = $this->request->param();
        $validate = new \think\Validate([
            ['daytime', 'date|dateFormat:Y-m-d'],
            ['page', 'number'],
            ['pagenum', 'number|<=:1000']
        ]);
        if (!$validate->check($post)) {
            $this->json_error('提交失败：' . $validate->getError());
        }

        $daytime = $this->request->param('daytime', date('Y-m-d'));
        $addressIds = $ticketModel->where('appointment_daytime', strtotime($daytime))->distinct(true)->column('address_id');

        $where = [];
        $where['id'] = ['in', $addressIds];
        $pagenum = $this->request->param('pagenum', 20, 'intval');
        $datalist = (new Address())->where($where)->paginate($pagenum, true);
        if (empty($datalist)) {
            $this->json_error("没有数据");
        }
        foreach ($datalist as $key => $item) {
            $item['thumb_url'] = geturl($item->thumb, '', true);
            $datalist[$key] = $item;
        }
        $this->json_success("查询成功", $datalist);
    }
}